@@ -9,22 +9,50 @@ use File::Find ();
 use File::Path ();
 use File::Basename ();
 use File::Spec ();
+use Data::Dumper ();
 
 sub new {
   my $package = shift;
-  my $self = bless {
-		    # Don't save these defaults in config_dir.
-		    build_script => 'Build',
-		    config_dir => '_build',
-		    @_,
-		    new_cleanup => {},
-		   }, $package;
-  
-  my ($action, $args) = $self->cull_args(@ARGV);
-  die "Too early to specify a build action '$action'.  Do ./$self->{build_script} $action instead.\n"
+  my %input = @_;
+
+  my $args   = delete $input{args}   || {};
+  my $config = delete $input{config} || {};
+
+  my ($action, $cmd_args) = __PACKAGE__->cull_args(@ARGV);
+  die "Too early to specify a build action '$action'.  Do 'Build $action' instead.\n"
     if $action;
 
-  $self->{args} = {%Config, @_, %$args};
+  my $cmd_config;
+  if ($cmd_args->{config}) {
+    # XXX need to hashify this string better (deal with quoted whitespace)
+    $cmd_config->{$1} = $2 while $cmd_args->{config} =~ /(\w+)=(\S+)/;
+  } else {
+    $cmd_config = {};
+  }
+  delete $cmd_args->{config};
+
+  # Extract our 'properties' from $cmd_args, the rest are put in 'args'
+  my $cmd_properties = {};
+  foreach my $key (keys %$cmd_args) {
+    $cmd_properties = delete $cmd_args->{$key} if __PACKAGE__->valid_property($key);
+  }
+
+  # 'args' are arbitrary user args.
+  # 'config' is Config.pm and its overridden values.
+  # 'properties' is stuff Module::Build needs in order to work.  They get saved in _build/.
+  # Anything else in $self doesn't get saved.
+
+  my $self = bless {
+		    args => {%$args, %$cmd_args},
+		    config => {%Config, %$config, %$cmd_config},
+		    properties => {
+				   build_script => 'Build',
+				   config_dir => '_build',
+				   %input,
+				   %$cmd_properties,
+				  },
+		    new_cleanup => {},
+		   }, $package;
 
   $self->check_manifest;
   $self->check_prereq;
@@ -44,19 +72,76 @@ sub resume {
   return $self;
 }
 
+{
+  # XXX huge hack alert - will revisit this later
+  my %valid_properties = map {$_ => 1}
+    qw(
+       module_name
+       module_version
+       module_version_from
+       prereq
+       config_dir
+       build_script
+       debugger
+       verbose
+       c_source
+       autosplit
+      );
+
+  sub valid_property { exists $valid_properties{$_[1]} }
+}
+
+# XXX Problem - if Module::Build is loaded from a different directory,
+# it'll look for (and perhaps destroy/create) a _build directory.
+sub subclass {
+  my ($pack, %opts) = @_;
+
+  my $build_dir = '_build'; # XXX The _build directory is ostensibly settable by the user.  Shouldn't hard-code here.
+  $pack->delete_filetree($build_dir) if -e $build_dir;
+
+  die "Must provide 'code' or 'class' option to subclass()\n"
+    unless $opts{code} or $opts{class};
+
+  $opts{code}  ||= '';
+  $opts{class} ||= 'MyModuleBuilder';
+  
+  my $filename = File::Spec->catfile($build_dir, 'lib', split '::', $opts{class}) . '.pm';
+  my $filedir  = File::Basename::dirname($filename);
+  print "Creating custom builder $filename in $filedir\n";
+  
+  File::Path::mkpath($filedir);
+  die "Can't create directory $filedir: $!" unless -d $filedir;
+  
+  open my($fh), ">$filename" or die "Can't create $filename: $!";
+  print $fh <<EOF;
+package $opts{class};
+use Module::Build;
+\@ISA = qw(Module::Build);
+$opts{code}
+1;
+EOF
+  close $fh;
+  
+  push @INC, File::Spec->catdir($build_dir, 'lib');
+  eval "use $opts{class}";
+  die $@ if $@;
+
+  return $opts{class};
+}
+
 sub find_version {
   my ($self) = @_;
-  return if exists $self->{args}{module_version};
+  return if exists $self->{properties}{module_version};
   
-  if (exists $self->{args}{module_version_from}) {
-    my $version = $self->version_from_file($self->{args}{module_version_from});
-    $self->{args}{module_version} = $version;
-    delete $self->{args}{module_version_from};
+  if (exists $self->{properties}{module_version_from}) {
+    my $version = $self->version_from_file($self->{properties}{module_version_from});
+    $self->{properties}{module_version} = $version;
+    delete $self->{properties}{module_version_from};
   } else {
     # Try to find the version in 'module_name'
-    my $chief_file = $self->module_name_to_file($self->{args}{module_name});
-    die "Can't find module '$self->{args}{module_name}' for version check" unless defined $chief_file;
-    $self->{args}{module_version} = $self->version_from_file($chief_file);
+    my $chief_file = $self->module_name_to_file($self->{properties}{module_name});
+    die "Can't find module '$self->{properties}{module_name}' for version check" unless defined $chief_file;
+    $self->{properties}{module_version} = $self->version_from_file($chief_file);
   }
 }
 
@@ -114,8 +199,8 @@ sub write_cleanup {
 }
 
 sub config_file {
-  my ($self) = @_;
-  return File::Spec->catfile($self->{config_dir}, $_[1]);
+  my $self = shift;
+  return File::Spec->catfile($self->{properties}{config_dir}, @_);
 }
 
 sub read_config {
@@ -123,14 +208,9 @@ sub read_config {
   
   my $file = $self->config_file('build_params');
   open my $fh, $file or die "Can't read '$file': $!";
-  
-  while (<$fh>) {
-    if (/^(\w+)$/) {
-      $self->{args}{$1} = undef;
-    } elsif (/^(\w+)=(.*)/) {
-      $self->{args}{$1} = $2;
-    }
-  }
+  my $ref = eval do {local $/; <$fh>};
+  die if $@;
+  ($self->{args}, $self->{config}, $self->{properties}) = @$ref;
   close $fh;
   
   my $cleanup_file = $self->config_file('cleanup');
@@ -146,78 +226,80 @@ sub read_config {
 sub write_config {
   my ($self) = @_;
   
-  $self->delete_filetree($self->{config_dir});
-  mkdir $self->{config_dir}, 0777
-    or die "Can't mkdir $self->{config_dir}: $!";
+  File::Path::mkpath($self->{properties}{config_dir});
+  -d $self->{properties}{config_dir} or die "Can't mkdir $self->{properties}{config_dir}: $!";
   
   my $file = $self->config_file('build_params');
   open my $fh, ">$file" or die "Can't create '$file': $!";
-  
-  foreach my $key (sort keys %{$self->{args}} ) {
-    if (!defined $self->{args}{$key}) {
-      print $fh "$key\n";
-
-    } elsif ($self->{args}{$key} =~ /\n/) {
-      warn "Sorry, can't handle newlines in config data '$key' (yet)\n";
-
-    } else {
-      print $fh "$key=$self->{args}{$key}\n";
-    }
-  }
+  local $Data::Dumper::Terse = 1;
+  print $fh Data::Dumper::Dumper([$self->{args}, $self->{config}, $self->{properties}]);
+  close $fh;
 }
 
 sub check_prereq {
   my $self = shift;
-  return 1 unless $self->{prereq};
+  return 1 unless $self->{properties}{prereq};
 
   my $pass = 1;
-  while (my ($modname, $spec) = each %{$self->{prereq}}) {
+  while (my ($modname, $spec) = each %{$self->{properties}{prereq}}) {
+    my $thispass = $self->check_installed_version($modname, $spec);
+    warn "WARNING: $@\n" unless $thispass;
+    $pass &&= $thispass;
+  }
 
-    my $file = $self->module_name_to_file($modname);
-    unless ($file) {
-      warn "WARNING: Prerequisite $modname isn't installed\n";
-      $pass = 0;
-      next;
-    }
-
-    my $version = $self->version_from_file($file);
-    if ($spec and !$version) {
-      warn "WARNING: Couldn't find a \$VERSION in prerequisite '$file'\n";
-      $pass = 0;
-      next;
-    }
-
-    my @conditions;
-    if ($spec =~ /^\s*([\w.]+)\s*$/) { # A plain number, maybe with dots, letters, and underscores
-      @conditions = (">= $spec");
-    } else {
-      @conditions = split /\s*,\s*/, $self->{prereq}{$modname};
-    }
-
-    foreach (@conditions) {
-      if ($_ !~ /^\s*  (<=?|>=?|==|!=)  \s*  [\w.]+  \s*$/x) {
-	warn "WARNING: Invalid prerequisite condition for $modname: $_\n";
-	next;
-      }
-      unless (eval "\$version $_") {
-	warn "WARNING: $modname version $version is installed, but we need Version($modname) $_\n";
-	$pass = 0;
-      }
-    }
+  if (!$pass) {
+    warn "ERRORS FOUND IN PREREQUISITES.  You may wish to install the versions ".
+         "of the modules indicated above before proceeding with this installation.\n";
   }
   return $pass;
 }
 
+sub check_installed_version {
+  my ($self, $modname, $spec) = @_;
+
+  my $file = $self->module_name_to_file($modname);
+  unless ($file) {
+    $@ = "Prerequisite $modname isn't installed";
+    return 0;
+  }
+
+  my $version = $self->version_from_file($file);
+  if ($spec and !$version) {
+    $@ = "Couldn't find a \$VERSION in prerequisite '$file'";
+    return 0;
+  }
+
+  my @conditions;
+  if ($spec =~ /^\s*([\w.]+)\s*$/) { # A plain number, maybe with dots, letters, and underscores
+    @conditions = (">= $spec");
+  } else {
+    @conditions = split /\s*,\s*/, $self->{properties}{prereq}{$modname};
+  }
+
+  foreach (@conditions) {
+    if ($_ !~ /^\s*  (<=?|>=?|==|!=)  \s*  [\w.]+  \s*$/x) {
+      $@ = "Invalid prerequisite condition for $modname: $_";
+      next;
+    }
+    unless (eval "\$version $_") {
+      $@ = "$modname version $version is installed, but we need version $_";
+      return 0;
+    }
+  }
+
+  return $version ? $version : '0 but true';
+}
+
 sub rm_previous_build_script {
   my $self = shift;
-  if (-e $self->{build_script}) {
-    print "Removing previous file '$self->{build_script}'\n";
-    unlink $self->{build_script} or die "Couldn't remove '$self->{build_script}': $!";
+  if (-e $self->{properties}{build_script}) {
+    print "Removing previous file '$self->{properties}{build_script}'\n";
+    unlink $self->{properties}{build_script} or die "Couldn't remove '$self->{properties}{build_script}': $!";
   }
 }
 
 sub make_build_script_executable {
-  chmod 0544, $_[0]->{build_script};
+  chmod 0544, $_[0]->{properties}{build_script};
 }
 
 sub print_build_script {
@@ -226,7 +308,7 @@ sub print_build_script {
   my $build_package = ref($self);
 
   my ($config_dir, $build_script, $build_dir) = 
-    ($self->{config_dir}, $self->{build_script},
+    ($self->{properties}{config_dir}, $self->{properties}{build_script},
      File::Spec->rel2abs(File::Basename::dirname($0)));  # XXX should be property of $self
 
   my @myINC = @INC;
@@ -237,16 +319,19 @@ sub print_build_script {
   my $quoted_INC = join ', ', map "'$_'", @myINC;
 
   print $fh <<EOF;
-$self->{args}{startperl} -w
+$self->{config}{startperl} -w
 
 BEGIN { \@INC = ($quoted_INC) }
 
 chdir('$build_dir') or die 'Cannot chdir to $build_dir: '.\$!;
 use $build_package;
 
+# This should have just enough arguments to be able to bootstrap the rest.
 my \$build = resume $build_package (
-  config_dir => '$config_dir',
-  build_script => '$build_script',
+  properties => {
+    config_dir => '$config_dir',
+    build_script => '$build_script',
+  },
 );
 eval {\$build->dispatch};
 my \$err = \$@;
@@ -261,9 +346,9 @@ sub create_build_script {
   
   $self->rm_previous_build_script;
 
-  print("Creating new '$self->{build_script}' script for ",
-	"'$self->{args}{module_name}' version '$self->{args}{module_version}'\n");
-  open my $fh, ">$self->{build_script}" or die "Can't create '$self->{build_script}': $!";
+  print("Creating new '$self->{properties}{build_script}' script for ",
+	"'$self->{properties}{module_name}' version '$self->{properties}{module_version}'\n");
+  open my $fh, ">$self->{properties}{build_script}" or die "Can't create '$self->{properties}{build_script}': $!";
   $self->print_build_script($fh);
   close $fh;
   
@@ -374,9 +459,9 @@ sub ACTION_test {
   
   $self->depends_on('build');
   
-  local $Test::Harness::switches = '-w -d' if $self->{args}{debugger};
-  local $Test::Harness::verbose = $self->{args}{verbose} || 0;
-  local $ENV{TEST_VERBOSE} = $self->{args}{verbose} || 0;
+  local $Test::Harness::switches = '-w -d' if $self->{properties}{debugger};
+  local $Test::Harness::verbose = $self->{properties}{verbose} || 0;
+  local $ENV{TEST_VERBOSE} = $self->{properties}{verbose} || 0;
 
   # Make sure we test the module in blib/
   {
@@ -390,6 +475,8 @@ sub ACTION_test {
   push @tests, 'test.pl'                          if -e 'test.pl';
   push @tests, @{$self->rscan_dir('t', qr{\.t$})} if -e 't' and -d _;
   if (@tests) {
+    # Work around a Test::Harness bug that loses the particular perl we're running under
+    local $^X = $self->{config}{perlpath} unless $Test::Harness::VERSION gt '2.01';
     Test::Harness::runtests(@tests);
   } else {
     print("No tests defined.\n");
@@ -404,16 +491,16 @@ sub ACTION_test {
 
 sub ACTION_testdb {
   my ($self) = @_;
-  local $self->{args}{debugger} = 1;
+  local $self->{properties}{debugger} = 1;
   $self->depends_on('test');
 }
 
 sub ACTION_build {
   my ($self) = @_;
   
-  if ($self->{args}{c_source}) {
-    push @{$self->{include_dirs}}, $self->{args}{c_source};
-    my $files = $self->rscan_dir($self->{args}{c_source}, qr{\.(c|PL)$});
+  if ($self->{properties}{c_source}) {
+    push @{$self->{include_dirs}}, $self->{properties}{c_source};
+    my $files = $self->rscan_dir($self->{properties}{c_source}, qr{\.(c|PL)$});
     foreach my $file (@$files) {
       if ($file =~ /c$/) {
 	push @{$self->{objects}}, $self->compile_c($file);
@@ -454,7 +541,7 @@ sub ACTION_clean {
 sub ACTION_realclean {
   my ($self) = @_;
   $self->depends_on('clean');
-  $self->delete_filetree($self->{config_dir}, $self->{build_script});
+  $self->delete_filetree($self->{properties}{config_dir}, $self->{properties}{build_script});
 }
 
 sub ACTION_dist {
@@ -528,8 +615,8 @@ sub ACTION_manifest {
 sub dist_dir {
   my ($self) = @_;
 
-  (my $dist_dir = $self->{args}{module_name}) =~ s/::/-/;
-  return "$dist_dir-$self->{args}{module_version}";
+  (my $dist_dir = $self->{properties}{module_name}) =~ s/::/-/;
+  return "$dist_dir-$self->{properties}{module_version}";
 }
 
 sub make_tarball {
@@ -545,8 +632,8 @@ sub install_map {
   my ($self, $blib) = @_;
   my $lib  = File::Spec->catfile($blib,'lib');
   my $arch = File::Spec->catfile($blib,'arch');
-  return {$lib  => $self->{args}{sitelib},
-	  $arch => $self->{args}{sitearch},
+  return {$lib  => $self->{config}{sitelib},
+	  $arch => $self->{config}{sitearch},
 	  read  => ''};  # To keep ExtUtils::Install quiet
 }
 
@@ -587,8 +674,8 @@ sub lib_to_blib {
   # Create $to/arch to keep blib.pm happy (what a load of hooie!)
   File::Path::mkpath( File::Spec->catdir($to, 'arch') );
 
-  if ($self->{args}{autosplit}) {
-    $self->autosplit_file($self->{args}{autosplit}, $to);
+  if ($self->{properties}{autosplit}) {
+    $self->autosplit_file($self->{properties}{autosplit}, $to);
   }
   
   foreach my $file (@$files) {
@@ -618,18 +705,18 @@ sub autosplit_file {
 
 sub compile_c {
   my ($self, $file) = @_;
-  my $args = $self->{args}; # For convenience
+  my $cf = $self->{config}; # For convenience
 
   # File name, minus the suffix
   (my $file_base = $file) =~ s/\.[^.]+$//;
-  my $obj_file = "$file_base$args->{obj_ext}";
+  my $obj_file = "$file_base$cf->{obj_ext}";
   return $obj_file if $self->up_to_date($file, $obj_file);
   
   $self->add_to_cleanup($obj_file);
-  my $coredir = File::Spec->catdir($args->{archlib}, 'CORE');
+  my $coredir = File::Spec->catdir($cf->{archlib}, 'CORE');
   my $include_dirs = $self->{include_dirs} ? join ' ', map {"-I$_"} @{$self->{include_dirs}} : '';
-  $self->do_system("$args->{cc} $include_dirs -c $args->{ccflags} -I$coredir -o $obj_file $file")
-    or die "error building $args->{dlext} file from '$file'";
+  $self->do_system("$cf->{cc} $include_dirs -c $cf->{ccflags} -I$coredir -o $obj_file $file")
+    or die "error building $cf->{dlext} file from '$file'";
 
   return $obj_file;
 }
@@ -637,14 +724,14 @@ sub compile_c {
 sub run_perl_script {
   my ($self, $script, $preargs, $postargs) = @_;
   $preargs ||= '';   $postargs ||= '';
-  return $self->do_system("$self->{args}{perl5} $preargs $script $postargs");
+  return $self->do_system("$self->{config}{perlpath} $preargs $script $postargs");
 }
 
 # A lot of this looks Unixy, but actually it may work fine on Windows.
 # I'll see what people tell me about their results.
 sub process_xs {
   my ($self, $file) = @_;
-  my $args = $self->{args}; # For convenience
+  my $cf = $self->{config}; # For convenience
 
   # File name, minus the suffix
   (my $file_base = $file) =~ s/\.[^.]+$//;
@@ -658,7 +745,7 @@ sub process_xs {
     my $typemap =  $self->module_name_to_file('ExtUtils::typemap');
     
     # XXX the '> $file_base.c' isn't really a post-arg, it's redirection.  Fix later.
-    $self->run_perl_script($xsubpp, "-I$args->{archlib} -I$args->{privlib}", 
+    $self->run_perl_script($xsubpp, "-I$cf->{archlib} -I$cf->{privlib}", 
 			   "-noprototypes -typemap '$typemap' $file > $file_base.c")
       or die "error building .c file from '$file'";
   }
@@ -685,13 +772,13 @@ sub process_xs {
   $self->copy_if_modified("$file_base.bs", $archdir, 1);
   
   # .o -> .(a|bundle)
-  my $lib_file = File::Spec->catfile($archdir, File::Basename::basename("$file_base.$args->{dlext}"));
-  unless ($self->up_to_date("$file_base$args->{obj_ext}", $lib_file)) {
-    my $linker_flags = $args->{extra_linker_flags} || '';
+  my $lib_file = File::Spec->catfile($archdir, File::Basename::basename("$file_base.$cf->{dlext}"));
+  unless ($self->up_to_date("$file_base$cf->{obj_ext}", $lib_file)) {
+    my $linker_flags = $cf->{extra_linker_flags} || '';
     my $objects = $self->{objects} || [];
-    $self->do_system("$args->{shrpenv} $args->{cc} $args->{lddlflags} -o $lib_file ".
-		     "$file_base$args->{obj_ext} @$objects $linker_flags")
-      or die "error building $args->{obj_ext} file from '$file_base.$args->{dlext}'";
+    $self->do_system("$cf->{shrpenv} $cf->{cc} $cf->{lddlflags} -o $lib_file ".
+		     "$file_base$cf->{obj_ext} @$objects $linker_flags")
+      or die "error building $file_base$cf->{obj_ext} from '$file_base.$cf->{dlext}'";
   }
 }
 
